@@ -10,6 +10,7 @@ use std::io::{BufReader, BufWriter, Write};
 use std::sync::Arc;
 use vrp_cli::core::solver::population::{get_default_population, Population};
 use vrp_cli::extensions::solve::config::create_builder_from_config_file;
+use vrp_cli::scientific::tsplib::{TsplibProblem, TsplibSolution};
 use vrp_cli::{get_errors_serialized, get_locations_serialized};
 use vrp_core::models::problem::ObjectiveCost;
 use vrp_core::models::{Problem, Solution};
@@ -37,6 +38,7 @@ const SEARCH_MODE_ARG_NAME: &str = "search-mode";
 const PARALELLISM_ARG_NAME: &str = "parallelism";
 const HEURISTIC_ARG_NAME: &str = "heuristic";
 const EXPERIMENTAL_ARG_NAME: &str = "experimental";
+const ROUNDED_ARG_NAME: &str = "round";
 
 #[allow(clippy::type_complexity)]
 struct ProblemReader(pub Box<dyn Fn(File, Option<Vec<File>>) -> Result<Problem, String>>);
@@ -49,6 +51,7 @@ struct SolutionWriter(
         dyn Fn(
             &Problem,
             Solution,
+            f64,
             Option<Metrics>,
             BufWriter<Box<dyn Write>>,
             Option<BufWriter<Box<dyn Write>>>,
@@ -62,35 +65,49 @@ struct LocationWriter(pub Box<dyn Fn(File, BufWriter<Box<dyn Write>>) -> Result<
 #[allow(clippy::type_complexity)]
 type FormatMap<'a> = HashMap<&'a str, (ProblemReader, InitSolutionReader, SolutionWriter, LocationWriter)>;
 
-fn add_scientific(formats: &mut FormatMap, random: Arc<dyn Random + Send + Sync>) {
+fn add_scientific(formats: &mut FormatMap, matches: &ArgMatches, random: Arc<dyn Random + Send + Sync>) {
     if cfg!(feature = "scientific-format") {
         use vrp_scientific::lilim::{LilimProblem, LilimSolution};
         use vrp_scientific::solomon::read_init_solution as read_init_solomon;
         use vrp_scientific::solomon::{SolomonProblem, SolomonSolution};
 
+        let is_rounded = matches.is_present(ROUNDED_ARG_NAME);
+
         formats.insert(
             "solomon",
             (
-                ProblemReader(Box::new(|problem: File, matrices: Option<Vec<File>>| {
+                ProblemReader(Box::new(move |problem: File, matrices: Option<Vec<File>>| {
                     assert!(matrices.is_none());
-                    BufReader::new(problem).read_solomon()
+                    BufReader::new(problem).read_solomon(is_rounded)
                 })),
                 InitSolutionReader(Box::new(move |file, problem| {
                     read_init_solomon(BufReader::new(file), problem, random.clone())
                 })),
-                SolutionWriter(Box::new(|_, solution, _, writer, _| solution.write_solomon(writer))),
+                SolutionWriter(Box::new(|_, solution, cost, _, writer, _| (&solution, cost).write_solomon(writer))),
                 LocationWriter(Box::new(|_, _| unimplemented!())),
             ),
         );
         formats.insert(
             "lilim",
             (
-                ProblemReader(Box::new(|problem: File, matrices: Option<Vec<File>>| {
+                ProblemReader(Box::new(move |problem: File, matrices: Option<Vec<File>>| {
                     assert!(matrices.is_none());
-                    BufReader::new(problem).read_lilim()
+                    BufReader::new(problem).read_lilim(is_rounded)
                 })),
                 InitSolutionReader(Box::new(|_file, _problem| unimplemented!())),
-                SolutionWriter(Box::new(|_, solution, _, writer, _| solution.write_lilim(writer))),
+                SolutionWriter(Box::new(|_, solution, cost, _, writer, _| (&solution, cost).write_lilim(writer))),
+                LocationWriter(Box::new(|_, _| unimplemented!())),
+            ),
+        );
+        formats.insert(
+            "tsplib",
+            (
+                ProblemReader(Box::new(move |problem: File, matrices: Option<Vec<File>>| {
+                    assert!(matrices.is_none());
+                    BufReader::new(problem).read_tsplib(is_rounded)
+                })),
+                InitSolutionReader(Box::new(|_file, _problem| unimplemented!())),
+                SolutionWriter(Box::new(|_, solution, cost, _, writer, _| (&solution, cost).write_tsplib(writer))),
                 LocationWriter(Box::new(|_, _| unimplemented!())),
             ),
         );
@@ -117,14 +134,14 @@ fn add_pragmatic(formats: &mut FormatMap, random: Arc<dyn Random + Send + Sync>)
             InitSolutionReader(Box::new(move |file, problem| {
                 read_init_pragmatic(BufReader::new(file), problem, random.clone())
             })),
-            SolutionWriter(Box::new(|problem, solution, metrics, default_writer, geojson_writer| {
+            SolutionWriter(Box::new(|problem, solution, cost, metrics, default_writer, geojson_writer| {
                 geojson_writer
-                    .map_or(Ok(()), |geojson_writer| solution.write_geo_json(problem, geojson_writer))
+                    .map_or(Ok(()), |geojson_writer| (&solution, cost).write_geo_json(problem, geojson_writer))
                     .and_then(|_| {
                         if let Some(metrics) = metrics {
-                            (solution, metrics).write_pragmatic_json(problem, default_writer)
+                            (&solution, cost, &metrics).write_pragmatic_json(problem, default_writer)
                         } else {
-                            solution.write_pragmatic_json(problem, default_writer)
+                            (&solution, cost).write_pragmatic_json(problem, default_writer)
                         }
                     })
             })),
@@ -139,10 +156,10 @@ fn add_pragmatic(formats: &mut FormatMap, random: Arc<dyn Random + Send + Sync>)
     );
 }
 
-fn get_formats<'a>(random: Arc<dyn Random + Send + Sync>) -> FormatMap<'a> {
+fn get_formats<'a>(matches: &ArgMatches, random: Arc<dyn Random + Send + Sync>) -> FormatMap<'a> {
     let mut formats = FormatMap::default();
 
-    add_scientific(&mut formats, random.clone());
+    add_scientific(&mut formats, matches, random.clone());
     add_pragmatic(&mut formats, random);
 
     formats
@@ -155,7 +172,7 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
             Arg::with_name(FORMAT_ARG_NAME)
                 .help("Specifies the problem type")
                 .required(true)
-                .possible_values(&["solomon", "lilim", "pragmatic"])
+                .possible_values(&["solomon", "lilim", "tsplib", "pragmatic"])
                 .index(1),
         )
         .arg(Arg::with_name(PROBLEM_ARG_NAME).help("Sets the problem file to use").required(true).index(2))
@@ -287,6 +304,13 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
                 .required(false)
                 .takes_value(false),
         )
+        .arg(
+            Arg::with_name(ROUNDED_ARG_NAME)
+                .help("Specifies whether costs are rounded. Applicable only for scientific formats.")
+                .long(ROUNDED_ARG_NAME)
+                .required(false)
+                .takes_value(false),
+        )
 }
 
 /// Runs solver commands.
@@ -296,7 +320,7 @@ pub fn run_solve(
 ) -> Result<(), String> {
     let environment = get_environment(matches)?;
 
-    let formats = get_formats(environment.random.clone());
+    let formats = get_formats(matches, environment.random.clone());
 
     // required
     let problem_path = matches.value_of(PROBLEM_ARG_NAME).unwrap();
@@ -360,13 +384,13 @@ pub fn run_solve(
                                 .with_hyper(get_heuristic(matches, problem.clone(), environment)?)
                         };
 
-                        let (solution, _, metrics) = builder
+                        let (solution, cost, metrics) = builder
                             .with_init_solutions(solutions, init_size)
                             .build()
                             .and_then(|solver| solver.solve())
                             .map_err(|err| format!("cannot find any solution: '{}'", err))?;
 
-                        solution_writer.0(&problem, solution, metrics, out_buffer, geo_buffer).unwrap();
+                        solution_writer.0(&problem, solution, cost, metrics, out_buffer, geo_buffer).unwrap();
 
                         if is_check_requested {
                             check_pragmatic_solution_with_args(matches)?;
